/* Project 2: 3D Visualization
 by Stejara Dinulescu
 MAT 259A, George Legrady
 
 DESCRIPTION: This is a 3D visualization of check-out counts over time for books whose titles include the terms "generational" or "embodied" and "memory".
 CURRENT STATE: This program brings in data results from the following mySQL query: 
 
                SELECT title, year(cin), month(cin), count(cin)
                FROM spl_2016.inraw 
                WHERE (title like '%memory%' OR title like '%memories%') AND 
                (title like '%embod%' OR title like '%generation%' or 
                 title like '%body%' OR title like '%famil%' or 
                 title like '%cognit%' OR title like '%autonom%')
                GROUP BY title, year(cin), month(cin);
 
                Each sphere is a unique title, and its movement through time is driven by its monthly check-in frequency.
 */

import peasy.*;
import controlP5.*;
PeasyCam cam;

Table table;

int NUM_AGENTS = 0; // this will increment as we read in csv rows
int startYear = 2006;
int endYear = 2021;
int TIME_POINTS = 12 * (endYear - startYear) + 13; // how many months in this period
int time_index = 0;
int MAX_COUNT = 0;
float system_rate; // this will turn into a slider
int unique_entry = 0;
int rowCount;

ArrayList<Entry> entries = new ArrayList<Entry>();
ControlP5 cp5;

void setup() {
  size(1300, 1300, P3D);
  colorMode(HSB, 255);
  background(0);

  cp5 = new ControlP5(this);
  cam = new PeasyCam(this, width/4 - 100, height/4 - 100, 50, 500);

  table = loadTable("data.csv", "header");
  rowCount = table.getRowCount();
  
  println("we have " + TIME_POINTS + " total time points");
  // 0 1 2 3 4 5 6 7 8 .... // these are array indices
  // 01-2010 02-2010 03-2010 04-2010 .... // these are the months they correspond to
  // 0-11 = 2010; 12-23 = 2011; 24-35 = 2012; ....

  // data read-in from csv, store into entry object
  String pstr = ""; 
  Entry e;
  for (int i = 0; i < rowCount; i++) {
    String title = table.getRow(i).getString("title");
    int year = table.getRow(i).getInt("year(cin)"); 
    int month = table.getRow(i).getInt("month(cin)");
    int count = table.getRow(i).getInt("count(cin)");
    if (count > MAX_COUNT) { MAX_COUNT = count; }
    int t_index = (year - startYear) * 12 + month; 

    println("---------");
    println("title: " + title); 
    println("count = " + count + "; date = " + year + "-" + month + "; array index = " + t_index);
    
    //calculate what time index this value goes into
    if (t_index < TIME_POINTS && t_index >= 0) {
      if (!pstr.equals(title)) { 
        e = new Entry(NUM_AGENTS, title, year, month, count); // this will set all counts and positions to zero
        e.storeCounts(t_index, count); // store the first one
        entries.add(e);
        NUM_AGENTS++;
        pstr = title; 
      } else { // store the time points 
        entries.get(NUM_AGENTS-1).storeCounts(t_index, count); // store the rest of the counts
      }
    } else { 
      println("---------");
      println("!!!!!!!!!!!");
      println("data row not stored: " + year + "-" + month); 
      // this happens because data years are outside of bounds
    }
  }
  
  // by this time, NUM_AGENTS, TIME_POINTS, and MAX_COUNTS are all established and everything from table is stored
  println("---------");
  println("we have " + NUM_AGENTS + " agents in the query and " + TIME_POINTS + " time points");
  for (int i = 0; i < NUM_AGENTS; i++) {
   entries.get(i).mapPosition(); 
  }
  
  // slider creation
  cp5.addSlider("system_rate")
    .setRange(0.1, 30.0)
    .setPosition(100, 50)
    .setSize(100, 20)
    .setValue(15.0);
  cp5.addSlider("unique_entry")
    .setRange(0, NUM_AGENTS-1)
    .setPosition(100, 80)
    .setSize(100, 20)
    .setValue(0);
  cp5.setAutoDraw(false);

  //style functions
  strokeJoin(ROUND);
  lights();
  smooth();
  
  PFont f = createFont("font.ttf", 128);
  textFont(f);
}

void draw() {
  background(0, 255, 0);
  directionalLight(0, 0, 255, 0, 0, -1);
  directionalLight(0, 0, 255, 0, 500, 0);
  ambientLight(0, 0, 100);
  if (time_index >= 0 && time_index < TIME_POINTS) {
    int true_count = 0;
    for (int i = 0; i < NUM_AGENTS; i++) {
      if (entries.get(i).updatePosition(time_index)) { true_count++; } 
      entries.get(i).drawAgent();
      entries.get(i).drawAgentHistory(time_index);
      entries.get(i).drawLabels();
    }
    //println("true count = " + true_count);
    if (true_count >= NUM_AGENTS) {
      time_index += 1;
      true_count = 0;
      //println("next time point: ", time_index, NUM_AGENTS);
    }
  }
  drawAxes();
  gui();
}

void drawAxes() {
  pushMatrix();
  stroke(0, 0, 255);
  strokeWeight(3);
  line(0, 0, 0, 500, 0, 0); 
  line(0, 0, 0, 0, 500, 0); 
  line(0, 0, 0, 0, 0, 500);
  
  fill(0, 0, 255); // pure white
  textAlign(CENTER, CENTER);
  textSize(30);
  scale(0.5);
  text("Unique Library Entry", 150, -20, 0);
  rotateY(-radians(90));
  rotateZ(radians(90));
  text("Check In Counts", 150, 20, 0);
  rotateX(radians(90));
  rotateY(-radians(90));
  text("Time (in Months)", 150, 20, 0);
  popMatrix();
}

void keyPressed() {
  if (key == ' ') {
    background(0, 255, 0);
    time_index = 0;
  }  // reset the system 
}

void mousePressed() {
 if (mouseX >= 0 && mouseX <= 200 && mouseY >= 0 && mouseY <= 200) { cam.setActive(false); }
 else { cam.setActive(true); } 
}

void gui() {
  hint(DISABLE_DEPTH_TEST);
  cam.beginHUD();
  cp5.draw();
  cam.endHUD();
  hint(ENABLE_DEPTH_TEST); 
}
